<?php

namespace Andy\SimilarResources\XFRM\Pub\Controller;

use XF\Mvc\ParameterBag;

class ResourceItem extends XFCP_ResourceItem
{
    public function actionView(ParameterBag $params)
    { 
		//########################################
		// get similar resources
		//########################################
        
		// get parent		
		$parent = parent::actionView($params);
        
        // return parent action if this is a redirect or other non View response 
        if (!$parent instanceof \XF\Mvc\Reply\View)
        {
            return $parent;
        }
        
        // get options
        $options = \XF::options();

        // get options from Admin CP -> Options -> Similar resources -> Limit
        $limit = $options->similarResourcesLimit;
        
        // get options from Admin CP -> Options -> Similar resources -> Same category
        $sameCategory = $options->similarResourcesSameCategory;
        
        // get options from Admin CP -> Options -> Similar resources -> Exclude categories
        $excludeCategories = $options->similarResourcesExcludeCategories;
        
        // get options from Admin CP -> Options -> Similar resources -> Minimum word length
        $minimumWordLength = $options->similarResourcesMinimumWordLength;
        
        // get options from Admin CP -> Options -> Similar resources -> Stop words
        $stopWords = $options->similarResourcesStopWords;
        
        // get options from Admin CP -> Options -> Similar resources -> Multibyte
        $multibyte = $options->similarResourcesMultibyte;

        // get resource
        $resource = $this->assertViewableResource($params->resource_id);

        // get resourceId
        $resourceId = $resource->resource_id;
        
        // get resourceTitle
        $resourceTitle = $resource->title;
        
        // get resourceCategoryId
        $resourceCategoryId = $resource->resource_category_id;

        // check condition
        if (empty($resourceTitle))
        {
            return;
        }

		//########################################
		// define variables
		//########################################

        $searchWords = array();
        $searchWord1 = '';
        $searchWord2 = '';
        $searchWord3 = '';
        $results1 = array();
        $results2 = array();
        $results3 = array();
		$results4 = array();
		$results5 = array();
        
		//########################################
		// prepare search words
		//########################################
        
        // check condition
		if (!$multibyte)
		{
            $resourceTitle = strtolower($resourceTitle);
            $resourceTitle = preg_replace('/[\p{P}]/u', ' ', $resourceTitle);
			$resourceTitleArray = explode(' ', $resourceTitle);
		}
		
        // check condition
		if ($multibyte)
		{
            $resourceTitle = mb_strtolower($resourceTitle);
            $resourceTitle = preg_replace('/[\p{P}]/u', ' ', $resourceTitle);
			$resourceTitleArray = mb_split(' ', $resourceTitle);
		} 

		// create stopWordsArray
		$stopWordsArray = explode(' ', $stopWords);								
	
		// remove words from array
		foreach ($resourceTitleArray as $var)
		{
			if (!$multibyte)
			{
				if (!in_array(strtolower($var), $stopWordsArray))
				{		
					// verify minimumCommonWordLength			
					if (strlen($var) >= $minimumWordLength)
					{
						$searchWords[] = $var;
					}
				}
			}

			if ($multibyte)
			{
				if (!in_array(mb_strtolower($var), $stopWordsArray))
				{	
					// verify minimumCommonWordLength				
					if (mb_strlen($var) >= $minimumWordLength)
					{
						$searchWords[] = $var;
					}
				}
			}
		}

		// get count
		$count = count($searchWords);
   
		// check condition
		if (empty($count))
		{
			return $parent;
		}

		// continue we have a search word
		if ($count > 0)
		{				
			// get first search word
			$searchWord1 = $searchWords[0];
			
			if ($count > 1)
			{	
				// get second search word
				$searchWord2 = $searchWords[1];	
			}
			
			if ($count > 2)
			{	
				// get third search word
				$searchWord3 = $searchWords[2];	
			}							
		}
        
		//########################################
		// exclude categories
        //########################################
        
        if (!empty($excludeCategories))
        {
            // get excludeResourceCategoriesArray
            $excludeCategoriesArray = explode(',', $excludeCategories); 

            // foreach condition
            foreach ($excludeCategoriesArray AS $value)
            {
                $conditions[] = ['resource_category_id', '<>', $value];
            }
        }
		
		//########################################
		// get mysqlVersion
        //########################################
		
		// get mysqlVersion
		$mysqlVersion = \XF::app()->db()->getServerVersion();
		$mysqlVersion = substr($mysqlVersion, 0, 1);

        //########################################
        // search 1
        //########################################
        
        // get condition
        $conditions[] = ['resource_id', '<>', $resourceId];
    
		// check condition
		if (!empty($searchWord1) AND !empty($searchWord2) AND !empty($searchWord3))
		{
            // get limit1
			$limit1 = $limit;
			
			// get finder
			$finder = \XF::finder('XFRM:ResourceItem');
			
			// check condition
			if ($mysqlVersion >= 8)
			{
				$searchValue1 = '\'' . $finder->escapeExpression($searchWord1) . '\\\\b\'';
				$searchValue2 = '\'' . $finder->escapeExpression($searchWord2) . '\\\\b\'';
				$searchValue3 = '\'' . $finder->escapeExpression($searchWord3) . '\\\\b\'';
			}
			else
			{
				$searchValue1 = '\'[[:<:]]' . $finder->escapeExpression($searchWord1) . '[[:>:]]\'';
				$searchValue2 = '\'[[:<:]]' . $finder->escapeExpression($searchWord2) . '[[:>:]]\'';
				$searchValue3 = '\'[[:<:]]' . $finder->escapeExpression($searchWord3) . '[[:>:]]\'';
			}
            
            // check condition
            if ($sameCategory AND empty($excludeCategories))
            {
                $results1 = $finder
                    ->where($finder->expression('title RLIKE ' . $searchValue1))
                    ->where($finder->expression('title RLIKE ' . $searchValue2))
                    ->where($finder->expression('title RLIKE ' . $searchValue3))
                    ->where('resource_state', '=', 'visible')
                    ->where('resource_category_id', '=', $resourceCategoryId)
                    ->where($conditions)
                    ->order('title', 'ASC')
                    ->limit($limit1)
                    ->fetchRaw();
            }
            
            // check condition
            if (!$sameCategory AND empty($excludeCategories))
            {
                $results1 = $finder
                    ->where($finder->expression('title RLIKE ' . $searchValue1))
                    ->where($finder->expression('title RLIKE ' . $searchValue2))
                    ->where($finder->expression('title RLIKE ' . $searchValue3))
                    ->where('resource_state', '=', 'visible')
                    ->where($conditions)
                    ->order('title', 'ASC')
                    ->limit($limit1)
                    ->fetchRaw();
            }
            
            // check condition
            if (!empty($excludeCategories))
            {
                $results1 = $finder
                    ->where($finder->expression('title RLIKE ' . $searchValue1))
                    ->where($finder->expression('title RLIKE ' . $searchValue2))
                    ->where($finder->expression('title RLIKE ' . $searchValue3))
                    ->where('resource_state', '=', 'visible')
                    ->where($conditions)
                    ->order('title', 'ASC')
                    ->limit($limit1)
                    ->fetchRaw();
            }
		}

        //########################################
        // search 2
        //########################################

		// check condition
		if (!empty($searchWord1) AND !empty($searchWord2))
		{
			foreach ($results1 AS $k => $v)
			{
				$resourceId = $v['resource_id'];
				$conditions[] = ['resource_id', '<>', $resourceId];
			}

			// check condition
			if (empty($conditions))
			{
				$conditions = [];
			}

			// get count
			$count = count($results1); 
            
			// check condition
			if ($count < $limit AND is_numeric($count))
			{
				// get limit2
				$limit2 = $limit - $count;
				
				// get finder
				$finder = \XF::finder('XFRM:ResourceItem');
				
				// check condition
				if ($mysqlVersion >= 8)
				{
					$searchValue1 = '\'' . $finder->escapeExpression($searchWord1) . '\\\\b\'';
					$searchValue2 = '\'' . $finder->escapeExpression($searchWord2) . '\\\\b\'';
				}
				else
				{
					$searchValue1 = '\'[[:<:]]' . $finder->escapeExpression($searchWord1) . '[[:>:]]\'';
					$searchValue2 = '\'[[:<:]]' . $finder->escapeExpression($searchWord2) . '[[:>:]]\'';
				}
                
                // check condition
                if ($sameCategory AND empty($excludeCategories))
                {
                    $results2 = $finder
                        ->where($finder->expression('title RLIKE ' . $searchValue1))
                        ->where($finder->expression('title RLIKE ' . $searchValue2))
                        ->where('resource_state', '=', 'visible')
                        ->where('resource_category_id', '=', $resourceCategoryId)
                        ->where($conditions)
                        ->order('title', 'ASC')
                        ->limit($limit2)
                        ->fetchRaw();
                }

                // check condition
                if (!$sameCategory AND empty($excludeCategories))
                {
                    $results2 = $finder
                        ->where($finder->expression('title RLIKE ' . $searchValue1))
                        ->where($finder->expression('title RLIKE ' . $searchValue2))
                        ->where('resource_state', '=', 'visible')
                        ->where($conditions)
                        ->order('title', 'ASC')
                        ->limit($limit2)
                        ->fetchRaw();
                }

                // check condition
                if (!empty($excludeCategories))
                {
                    $results2 = $finder
                        ->where($finder->expression('title RLIKE ' . $searchValue1))
                        ->where($finder->expression('title RLIKE ' . $searchValue2))
                        ->where('resource_state', '=', 'visible')
                        ->where($conditions)
                        ->order('title', 'ASC')
                        ->limit($limit2)
                        ->fetchRaw();
                }    
            }
		}

        //########################################
        // search 3
        //########################################

		// check condition
		if (!empty($searchWord1))
		{
			// for each condition
			foreach ($results2 AS $k => $v)
			{
				$resourceId = $v['resource_id'];
				$conditions[] = ['resource_id', '<>', $resourceId];
			}
			
			// check condition
			if (empty($conditions))
			{
				$conditions = [];
			}

			// get count
			$count = count($results1) + count($results2); 
        
			// check condition
			if ($count < $limit AND is_numeric($count))
			{
				// get limit3
				$limit3 = $limit - $count;
				
				// get finder
				$finder = \XF::finder('XFRM:ResourceItem');
				
				// check condition
				if ($mysqlVersion >= 8)
				{
					$searchValue1 = '\'' . $finder->escapeExpression($searchWord1) . '\\\\b\'';
				}
				else
				{
					$searchValue1 = '\'[[:<:]]' . $finder->escapeExpression($searchWord1) . '[[:>:]]\'';
				}
                
                // check condition
                if ($sameCategory AND empty($excludeCategories))
                {
                    $results3 = $finder
                        ->where($finder->expression('title RLIKE ' . $searchValue1))
                        ->where('resource_state', '=', 'visible')
                        ->where('resource_category_id', '=', $resourceCategoryId)
                        ->where($conditions)
                        ->order('title', 'ASC')
                        ->limit($limit3)
                        ->fetchRaw();
                }

                // check condition
                if (!$sameCategory AND empty($excludeCategories))
                {
                    $results3 = $finder
                        ->where($finder->expression('title RLIKE ' . $searchValue1))
                        ->where('resource_state', '=', 'visible')
                        ->where($conditions)
                        ->order('title', 'ASC')
                        ->limit($limit3)
                        ->fetchRaw();
                }

                // check condition
                if (!empty($excludeCategories))
                {
                    $results3 = $finder
                        ->where($finder->expression('title RLIKE ' . $searchValue1))
                        ->where('resource_state', '=', 'visible')
                        ->where($conditions)
                        ->order('title', 'ASC')
                        ->limit($limit3)
                        ->fetchRaw();
                }
            }
		}
		
        //########################################
        // search 4
        //########################################

		// check condition
		if (!empty($searchWord2))
		{
			// for each condition
			foreach ($results3 AS $k => $v)
			{
				$resourceId = $v['resource_id'];
				$conditions[] = ['resource_id', '<>', $resourceId];
			}
			
			// check condition
			if (empty($conditions))
			{
				$conditions = [];
			}

			// get count
			$count = count($results1) + count($results2) + count($results3); 
            
			// check condition
			if ($count < $limit AND is_numeric($count))
			{
				// get limit 4
				$limit4 = $limit - $count;
				
				// get finder
				$finder = \XF::finder('XFRM:ResourceItem');
				
				// check condition
				if ($mysqlVersion >= 8)
				{
					$searchValue2 = '\'' . $finder->escapeExpression($searchWord2) . '\\\\b\'';
				}
				else
				{
					$searchValue2 = '\'[[:<:]]' . $finder->escapeExpression($searchWord2) . '[[:>:]]\'';
				}
                
                // check condition
                if ($sameCategory AND empty($excludeCategories))
                {
                    $results4 = $finder
                        ->where($finder->expression('title RLIKE ' . $searchValue2))
                        ->where('resource_state', '=', 'visible')
                        ->where('resource_category_id', '=', $resourceCategoryId)
                        ->where($conditions)
                        ->order('title', 'ASC')
                        ->limit($limit4)
                        ->fetchRaw();
                }

                // check condition
                if (!$sameCategory AND empty($excludeCategories))
                {
                    $results4 = $finder
                        ->where($finder->expression('title RLIKE ' . $searchValue2))
                        ->where('resource_state', '=', 'visible')
                        ->where($conditions)
                        ->order('title', 'ASC')
                        ->limit($limit4)
                        ->fetchRaw();
                }

                // check condition
                if (!empty($excludeCategories))
                {
                    $results4 = $finder
                        ->where($finder->expression('title RLIKE ' . $searchValue2))
                        ->where('resource_state', '=', 'visible')
                        ->where($conditions)
                        ->order('title', 'ASC')
                        ->limit($limit4)
                        ->fetchRaw();
                }
            }
		}
		
        //########################################
        // search 5
        //########################################

		// check condition
		if (!empty($searchWord3))
		{
			// for each condition
			foreach ($results4 AS $k => $v)
			{
				$resourceId = $v['resource_id'];
				$conditions[] = ['resource_id', '<>', $resourceId];
			}
			
			// check condition
			if (empty($conditions))
			{
				$conditions = [];
			}

			// get count
			$count = count($results1) + count($results2) + count($results3) + count($results4); 
            
			// check condition
			if ($count < $limit AND is_numeric($count))
			{
				// get limit5
				$limit5 = $limit - $count;
				
				// get finder
				$finder = \XF::finder('XFRM:ResourceItem');
				
				// check condition
				if ($mysqlVersion >= 8)
				{
					$searchValue3 = '\'' . $finder->escapeExpression($searchWord3) . '\\\\b\'';
				}
				else
				{
					$searchValue3 = '\'[[:<:]]' . $finder->escapeExpression($searchWord3) . '[[:>:]]\'';
				}
                
                // check condition
                if ($sameCategory AND empty($excludeCategories))
                {
                    $results5 = $finder
                        ->where($finder->expression('title RLIKE ' . $searchValue3))
                        ->where('resource_state', '=', 'visible')
                        ->where('resource_category_id', '=', $resourceCategoryId)
                        ->where($conditions)
                        ->order('title', 'ASC')
                        ->limit($limit5)
                        ->fetchRaw();
                }

                // check condition
                if (!$sameCategory AND empty($excludeCategories))
                {
                    $results5 = $finder
                        ->where($finder->expression('title RLIKE ' . $searchValue3))
                        ->where('resource_state', '=', 'visible')
                        ->where($conditions)
                        ->order('title', 'ASC')
                        ->limit($limit5)
                        ->fetchRaw();
                }

                // check condition
                if (!empty($excludeCategories))
                {
                    $results5 = $finder
                        ->where($finder->expression('title RLIKE ' . $searchValue3))
                        ->where('resource_state', '=', 'visible')
                        ->where($conditions)
                        ->order('title', 'ASC')
                        ->limit($limit5)
                        ->fetchRaw();
                }
            }
		}

        // get results
        $results = array_merge($results1, $results2, $results3, $results4, $results5);
     
        // check condition
        if (!empty($results))
        {
            // foreach condition
            foreach ($results AS $result)
            {
                $ids[] = $result['resource_id'];
            }
        
            // get similarResources
            $finder = \XF::finder('XFRM:ResourceItem')
                ->whereIds($ids);
            $similarResources = $finder->fetch();
            $similarResources = $similarResources->sortByList($ids);
            
            // set parent
            $parent->setParams([
				'similarResources' => $similarResources
            ]);
        }	
		
		// return parent
		return $parent;	
	}
}